require_relative '../../inc/ukrsol/server/rtt_plugin_server'

class TimeTrackerAdministratorAppController < ApplicationController
  # unloadable
  ActiveRecord::Base.include_root_in_json = false
  RETURN_BACK_DATA_FIELD = 'request_id'
  CLEAN_FREQUENCY_FACTOR = 1000

  if Redmine::VERSION::MAJOR <= 2
    before_filter :check_if_user_admin, :except => [:check_plugin, :get_user_settings, :add_time, :statuses_for_issue, :update_user_state, :actions]
    before_filter :find_issue, :only => [:statuses_for_issue]
    before_filter :check_if_user_logged_in, :only => [:check_plugin, :get_user_settings, :add_time, :statuses_for_issue, :update_user_state, :actions]
  else
    before_action :check_if_user_admin, :except => [:check_plugin, :get_user_settings, :add_time, :statuses_for_issue, :update_user_state, :actions]
    before_action :find_issue, :only => [:statuses_for_issue]
    before_action :check_if_user_logged_in, :only => [:check_plugin, :get_user_settings, :add_time, :statuses_for_issue, :update_user_state, :actions]
    self.main_menu = false
  end

  if Redmine::VERSION::MAJOR == 1 and Redmine::VERSION::MINOR <= 3
    accept_key_auth :check_plugin, :get_user_settings, :add_time, :statuses_for_issue, :update_user_state, :actions
  else
    accept_api_auth :check_plugin, :get_user_settings, :add_time, :statuses_for_issue, :update_user_state, :actions
  end

  def check_if_user_admin
    deny_access unless User.current.admin
  end

  def check_if_user_logged_in
    deny_access unless User.current.status == 1
  end

  def start_server

    # Do not start server if plugin disabled
    unless Setting.plugin_redmine_time_tracker_administrator['plugin_enabled']
      return
    end

    # settings = Setting.plugin_redmine_time_tracker_administrator
    # socket_port = settings["socket_port"]
    # TODO: temporary hardcode socket port value
    socket_port = 47123

    # WORKING VARIANT !!!
    Thread.new do
      begin
        if defined?($rtta_server) && $rtta_server != nil
          ::Ukrsol::Server::RttPluginServer.instance.run('0.0.0.0', socket_port)
        else
          $rtta_server = ::Ukrsol::Server::RttPluginServer.instance
          $rtta_server.set_auth_url request.base_url+rtta_events_authenticate_path
          $rtta_server.run('0.0.0.0', socket_port)
        end
      rescue StandardError => e
        Thread.exit
      end
    end
  end

  ##############################
  #RTT API
  ##############################
  def check_plugin
    begin
      # Check if plugin enabled
      if Setting.plugin_redmine_time_tracker_administrator['plugin_enabled']

        # Get proper settings
        user_individual_settings = RttaSetting.where(item_type: 'user', item_id: User.current.id).first
        default_settings = RttaSetting.where(item_type: 'default').first

        # Replace some user settings with default settings
        if user_individual_settings
          user_individual_settings = user_individual_settings.to_rtt_app_format
          default_settings = default_settings.to_rtt_app_format

          user_individual_settings[:settings]['PRIORITIES_ORDER'] = default_settings[:settings]['PRIORITIES_ORDER']
          user_individual_settings[:settings]['PRIORITIES_COLORS'] = default_settings[:settings]['PRIORITIES_COLORS']
          user_individual_settings[:read_only_state]['PRIORITIES_ORDER'] = default_settings[:read_only_state]['PRIORITIES_ORDER']
          user_individual_settings[:read_only_state]['PRIORITIES_COLORS'] = default_settings[:read_only_state]['PRIORITIES_COLORS']

          # settings = user_individual_settings
          settings = default_settings.merge(user_individual_settings) {|key, a_val, b_val| a_val.merge b_val }
        else
          settings = default_settings.to_rtt_app_format
        end

        # check for TIME_TRACKING_COMMENT_REQUIRED option
        unless settings[:settings].key?('TIME_TRACKING_COMMENT_REQUIRED')
          settings[:settings]['TIME_TRACKING_COMMENT_REQUIRED'] = false
          settings[:read_only_state]['TIME_TRACKING_COMMENT_REQUIRED'] = false
        end

        # Add client IP info
        settings[:client_ip] = request.env["REMOTE_ADDR"]

        # Add plugin settings
        # settings[:socket_port] = Setting.plugin_redmine_time_tracker_administrator['socket_port']
        settings[:socket_port] = 47123
        settings[:key] = Setting.plugin_redmine_time_tracker_administrator['activation_key']

        # Add user info
        settings[:user] = User.current

        # Add users info
        settings[:users] = StaffManagerUsersHelper::get_all_users

        # Add plugin version info
        settings[:plugin_version] = '3.1.9'

        render :json => settings
      else
        render_404
      end
    rescue => e
      message = "RTTA. Check plugin (get settings) error: #{e.message}"
      Rails.logger.error message
      render :json => {message: message}
    end


    start_server
  end

  def get_user_settings
    begin
      settings_info = RttaSetting.where(item_type: 'user', item_id: User.current.id).first
      settings_info = RttaSetting.where(item_type: 'default').first unless settings_info

      default_settings_info = RttaSetting.where(item_type: 'default').first

      user_individual_settings = settings_info.to_rtt_app_format
      default_settings = default_settings_info.to_rtt_app_format

      result = default_settings.merge(user_individual_settings) {|key, a_val, b_val| a_val.merge b_val }

      # check for TIME_TRACKING_COMMENT_REQUIRED option
      unless result[:settings].key?('TIME_TRACKING_COMMENT_REQUIRED')
        result[:settings]['TIME_TRACKING_COMMENT_REQUIRED'] = false
        result[:read_only_state]['TIME_TRACKING_COMMENT_REQUIRED'] = false
      end

      # render :json => settings_info.to_rtt_app_format
      render :json => result
    rescue => e
      message = "RTTA. Get user settings error: #{e.message}"
      Rails.logger.error message
      render :json => {errors: [message]}
    end
  end

  def actions
    begin
      require_relative './validations/update_user_states_data'

      result = {}
      result[:add_time] = add_time(params[:add_time]) if params.key?(:add_time)
      result[:time_entries] = time_entries(params[:time_entries]) if params.key?(:time_entries)
      result[:status] = update_user_state(params[:status]) if params.key?(:status)
      result[:custom_fields] = get_custom_fields if (params.key?(:custom_fields) && params[:custom_fields].key?('request') && params[:custom_fields]['request'] === 'get')
      render :json => result
    rescue => e
      render :json => {errors: [e.message]}
    end
  end

  def time_entries(data)
    begin
      case data['request']
      when 'get_spent_time'

        # Check if need spent time for issue or for project
        if data.key?("issue_id")
          issue = Issue.where(id: data["issue_id"]).first
          return {spent_time: issue.total_spent_hours.round(2)}
        elsif data.key?("project_id")
          # project = Project.where(id: data["project_id"]).first
          # cond = project.project_condition(Setting.display_subprojects_issues?)
          # total_hours = TimeEntry.visible.where(cond).sum(:hours).to_f.round(2)
          # return {spent_time: total_hours}
          return {spent_time: get_project_total_time_spent(data["project_id"])}
        else
          raise "issue_id or project_id not provided"
        end

      else
        message = "Rtta Server. time_entries error, no valid 'request' parameter"
        Rails.logger.error message
        return {errors: [message]}
      end
    rescue StandardError => e
      message = "Rtta Server. get time_entries info error: #{e.message}"
      Rails.logger.error message
      return {errors: [message]}
    end
    # return {list: "test"}
  end

  def get_project_total_time_spent(project_id)
    project = Project.where(id: project_id).first
    cond = project.project_condition(Setting.display_subprojects_issues?)
    TimeEntry.visible.where(cond).sum(:hours).to_f.round(2)
  end

  def statuses_for_issue
    begin
      render :json => {issue_statuses: @issue.new_statuses_allowed_to(User.current)}
    rescue => e
      message = "RTTA. Statuses for issue error: #{e.message}"
      Rails.logger.error message
      render :json => {errors: [message]}
    end
  end

  ############################
  # private functions
  # ##########################
  def add_time(data)
    begin
      post_data = data
      h = post_data["spent_time"]
      h = (h.is_a?(String) ? (h.to_hours || h) : h)
      if h == 0
        result = {:time_entry => nil}
      else
        update_time_result = update_ticket_time(client = nil, post_data)
        if update_time_result.key?(:data) && update_time_result[:data].is_a?(TimeEntry)
          total_time_spent = "%0.2f" % (data.key?("project_id") ? get_project_total_time_spent(data["project_id"]) : [update_time_result[:data].issue.total_spent_hours])
          result = {
              :time_entry => update_time_result[:data].as_json,
              :total_spent_time => total_time_spent.to_f,
              :duplicate => update_time_result.key?(:duplicate)
          }
        else
          result = update_time_result[:data]
        end
      end

      # # Send result back to client
      result[RETURN_BACK_DATA_FIELD] = post_data[RETURN_BACK_DATA_FIELD] if post_data.key?(RETURN_BACK_DATA_FIELD)

      # update_user_state(post_data)

      # Clean request_ids approximately one per CLEAN_FREQUENCY_FACTOR times
      RttaAppRequest.clean_old if rand(CLEAN_FREQUENCY_FACTOR) == 0

      return result
    rescue StandardError => e
      message = "Rtta Server. update ticket time handler error: #{e.message}"
      Rails.logger.error message
      return {errors: [message]}
    end
  end

  def save_request_data(time_entry, params, user_current)
    key_param_name = params.key?("project_id") ? "project_id" : "issue_id"
    # Find existed data. Or create new
    # rtta_app_request = RttaAppRequest.where(user_id: user_current.id, issue_id: time_entry.issue.id).first
    rtta_app_request = RttaAppRequest.where(user_id: user_current.id, key_param_name.to_sym => params[key_param_name]).first
    rtta_app_request = RttaAppRequest.new(
        user_id: user_current.id,
        key_param_name.to_sym => params[key_param_name]
    # request_ids: [params[RETURN_BACK_DATA_FIELD]].to_json,
        ) unless rtta_app_request

    # Add request id to array
    rtta_app_request.add_request_id params[RETURN_BACK_DATA_FIELD]
    rtta_app_request.save
  end

  def update_ticket_time(client, params)
    user_current = User.current

    # issue = params.key?("issue_id") ? Issue.where(id: params["issue_id"]).first : nil
    # project = params.key?("project_id") ? Project.where(id: params["project_id"]).first : nil


    # Project key entity
    if params.key?("project_id")
      key_param_name = "project_id"
      project = Project.where(id: params["project_id"]).first
      # Project not found
      unless project
        return {data: {errors: ['Project not found']}}
      end
    else
      # Issue key entity
      key_param_name = "issue_id"
      issue = Issue.where(id: params["issue_id"]).first
      # Issue not found
      unless issue
        return {data: {errors: ['Issue not found']}}
      end
    end


    # Check if time was not added before (check request_id)
    existed_request_id_condition = RttaAppRequest
                                       .where(user_id: user_current.id, key_param_name.to_sym => params[key_param_name])
                                       .where("#{RttaAppRequest.table_name}.request_ids LIKE ?", "%#{params[RETURN_BACK_DATA_FIELD]}%")
    if existed_request_id_condition.exists?
      # Time already added, just return time entry without saving.
      if "project_id" == key_param_name
        return {
            data: TimeEntry.order("id DESC").where(
                spent_on: params["date"],
                project_id: params["project_id"],
                issue_id: nil,
                user_id: user_current.id,
                comments: params["comment"],
                activity_id: params["activity_id"],
                ).first,
            duplicate: true
        }
      else
        return {
            data: TimeEntry.order("id DESC").where(
                spent_on: params["date"],
                issue_id: params["issue_id"],
                user_id: user_current.id,
                comments: params["comment"],
                activity_id: params["activity_id"],
                ).first,
            duplicate: true
        }
      end
    end

    # Try to find existed time entry
    if "project_id" == key_param_name
      time_entry = TimeEntry.order("id DESC").where(
        spent_on: params["date"],
        project_id: params["project_id"],
        issue_id: nil,
        user_id: user_current.id,
        comments: params["comment"],
        activity_id: params["activity_id"],
        ).first
    else
      time_entry = TimeEntry.order("id DESC").where(
        spent_on: params["date"],
        issue_id: params[key_param_name],
        user_id: user_current.id,
        comments: params["comment"],
        activity_id: params["activity_id"],
        ).first
    end

    # Update existed time entry or create new
    if time_entry && (!params.key?("custom_field_values") || !custom_field_values_changed?(time_entry, params['custom_field_values']))
      h = params["spent_time"]
      begin
        time_entry.hours = time_entry.hours.to_f + (h.is_a?(String) ? (h.to_hours || h) : h)
      rescue TypeError => e
        # return {errors: [l(:staff_manager_incorrect_spent_time) % [issue.id]]}
        return {data: {errors: ['Incorrect spent time.' % [issue.id]]}}
      end
    else
      # Create new
      if "project_id" == key_param_name
        time_entry = TimeEntry.new(
          # :issue => issue,
          :project => project,
          :user => user_current,
          :spent_on => params["date"],
          :hours => params["spent_time"].to_f,
          :comments => params["comment"],
          :activity_id => params["activity_id"],
          )
      else
        time_entry = TimeEntry.new(
          :issue => issue,
          :project => issue ? issue.project : nil,
          :user => user_current,
          :spent_on => params["date"],
          :hours => params["spent_time"].to_f,
          :comments => params["comment"],
          :activity_id => params["activity_id"],
          )
      end
      time_entry.custom_field_values = params["custom_field_values"] if params.key?("custom_field_values")
    end

    # Check authorization
    if time_entry.project && !user_current.allowed_to?(:log_time, time_entry.project)
      # return {errors: [l(:rtta_not_allowed_add_time_entry) % [issue.id]]}
      return {data: {errors: ['You are not allowed to add time to task #%s. Contact redmine administrator to get access.' % [issue.id]]}}
    end

    # Save time entry and return response
    if time_entry.save
      # Save request data (request_id)
      save_request_data time_entry, params, user_current
      {data: time_entry}
    else
      {data: {errors: [time_entry.errors.to_json]}}
    end
  end

  # def update_ticket_time(client, params)
  #   user_current = User.current
  #
  #   issue = Issue.where(id: params["issue_id"]).first
  #
  #   # Issue not found
  #   unless issue
  #     # return {errors: [l(:rtta_issue_not_found)]}
  #     return {data: {errors: ['Issue not found']}}
  #   end
  #
  #
  #   # Check if time was not added before (check request_id)
  #   existed_request_id_condition = RttaAppRequest
  #                                      .where(user_id: user_current.id, issue_id: params["issue_id"])
  #                                      .where("#{RttaAppRequest.table_name}.request_ids LIKE ?", "%#{params[RETURN_BACK_DATA_FIELD]}%")
  #   if existed_request_id_condition.exists?
  #     # Time already added, just return time entry without saving.
  #     return {
  #         data: TimeEntry.order("id DESC").where(
  #             spent_on: params["date"],
  #             issue_id: params["issue_id"],
  #             user_id: user_current.id,
  #             comments: params["comment"],
  #             activity_id: params["activity_id"],
  #             ).first,
  #         duplicate: true
  #     }
  #   end
  #
  #   # Try to find existed time entry
  #   time_entry = TimeEntry.order("id DESC").where(
  #       spent_on: params["date"],
  #       issue_id: params["issue_id"],
  #       user_id: user_current.id,
  #       comments: params["comment"],
  #       activity_id: params["activity_id"],
  #       ).first
  #
  #   # Update existed time entry or create new
  #   if time_entry && (!params.key?("custom_field_values") || !custom_field_values_changed?(time_entry, params['custom_field_values']))
  #     h = params["spent_time"]
  #     begin
  #       time_entry.hours = time_entry.hours.to_f + (h.is_a?(String) ? (h.to_hours || h) : h)
  #     rescue TypeError => e
  #       # return {errors: [l(:staff_manager_incorrect_spent_time) % [issue.id]]}
  #       return {data: {errors: ['Incorrect spent time.' % [issue.id]]}}
  #     end
  #   else
  #     # Create new
  #     time_entry = TimeEntry.new(
  #         :issue => issue,
  #         :project => issue ? issue.project : nil,
  #         :user => user_current,
  #         :spent_on => params["date"],
  #         :hours => params["spent_time"].to_f,
  #         :comments => params["comment"],
  #         :activity_id => params["activity_id"],
  #         )
  #     time_entry.custom_field_values = params["custom_field_values"] if params.key?("custom_field_values")
  #   end
  #
  #   # Check authorization
  #   if time_entry.project && !user_current.allowed_to?(:log_time, time_entry.project)
  #     # return {errors: [l(:rtta_not_allowed_add_time_entry) % [issue.id]]}
  #     return {data: {errors: ['You are not allowed to add time to task #%s. Contact redmine administrator to get access.' % [issue.id]]}}
  #   end
  #
  #   # Save time entry and return response
  #   if time_entry.save
  #     # Save request data (request_id)
  #     save_request_data time_entry, params, user_current
  #     {data: time_entry}
  #   else
  #     {data: {errors: [time_entry.errors.to_json]}}
  #   end
  # end

  def update_user_state(data)
    begin
      update_user_state_data = Validations::UpdateUserStatesData.new(data)

      unless update_user_state_data.valid?
         return {errors: [update_user_state_data.errors.to_json]}
      end

      # return {errors: ["message #{update_user_state_data.issues_ids} #{update_user_state_data.issues_ids.length}"]}
      # condition = update_user_state_data.issues_ids.is_a?(Array) && update_user_state_data.issues_ids.length
      # return {errors: ["message #{!!condition}"]}

      # Update state by issues
      if update_user_state_data.issues_ids.is_a?(Array) && update_user_state_data.issues_ids.length > 0

        # Clear old user statuses
        old_user_states = update_user_state_data.projects_ids.length > 0 ? RttaUserState.where(user_id: User.current.id, project_id: nil).all : RttaUserState.where(user_id: User.current.id).all
        old_user_states.each do |user_state|
          unless update_user_state_data.issues_ids.include?(user_state.issue_id)
            user_state.destroy
          end
        end

        # Create new statuses for each issue_id or update old
        update_user_state_data.issues_ids.each do |issue_id|
          if RttaUserState.exists?(user_id: User.current.id, issue_id: issue_id)
            user_state = RttaUserState.where(user_id: User.current.id, issue_id: issue_id).first
            if data['status'] == 'paused' && user_state.status != 'paused'
              if Redmine::VERSION::MAJOR <= 4
                user_state.update_attributes(status: data["status"], status_order: StaffManagerStatus::ORDER[data["status"]], updated_at: Time.now, paused_at: Time.now)
              else
                user_state.update(status: data["status"], status_order: StaffManagerStatus::ORDER[data["status"]], updated_at: Time.now, paused_at: Time.now)
              end
            else
              if Redmine::VERSION::MAJOR <= 4
                user_state.update_attributes(status: data["status"], status_order: StaffManagerStatus::ORDER[data["status"]], updated_at: Time.now)
              else
                user_state.update(status: data["status"], status_order: StaffManagerStatus::ORDER[data["status"]], updated_at: Time.now)
              end
            end
          else
            RttaUserState.create!(user_id: User.current.id, issue_id: issue_id, status: data['status'], status_order: StaffManagerStatus::ORDER[data["status"]], paused_at: Time.now)
          end
        end
      end

      if update_user_state_data.projects_ids.is_a?(Array) && update_user_state_data.projects_ids.length > 0
        # Update state by projects
        # Clear old user statuses
        old_user_states = ((update_user_state_data.issues_ids.length > 0) ? RttaUserState.where(user_id: User.current.id, issue_id: nil).all : RttaUserState.where(user_id: User.current.id).all)
        old_user_states.each do |user_state|
         unless update_user_state_data.projects_ids.include?(user_state.project_id)
           user_state.destroy
         end
        end

        # Create new statuses for each project id or update old
        update_user_state_data.projects_ids.each do |project_id|
         if RttaUserState.exists?(user_id: User.current.id, project_id: project_id)
           user_state = RttaUserState.where(user_id: User.current.id, project_id: project_id).first
           if data['status'] == 'paused' && user_state.status != 'paused'
             if Redmine::VERSION::MAJOR <= 4
               user_state.update_attributes(status: data["status"], status_order: StaffManagerStatus::ORDER[data["status"]], updated_at: Time.now, paused_at: Time.now)
             else
               user_state.update(status: data["status"], status_order: StaffManagerStatus::ORDER[data["status"]], updated_at: Time.now, paused_at: Time.now)
             end
           else
             if Redmine::VERSION::MAJOR <= 4
               user_state.update_attributes(status: data["status"], status_order: StaffManagerStatus::ORDER[data["status"]], updated_at: Time.now)
             else
               user_state.update(status: data["status"], status_order: StaffManagerStatus::ORDER[data["status"]], updated_at: Time.now)
             end
           end
         else
           RttaUserState.create!(user_id: User.current.id, project_id: project_id, status: data['status'], status_order: StaffManagerStatus::ORDER[data["status"]], paused_at: Time.now)
         end
        end
      end

      if !(update_user_state_data.projects_ids.is_a?(Array) && update_user_state_data.projects_ids.length > 0) && !(update_user_state_data.projects_ids.is_a?(Array) && update_user_state_data.projects_ids.length > 0)
        # Update only statuses for all user states, or create new with given status
        new_user_state = RttaUserState.exists?(user_id: User.current.id) ? nil : RttaUserState.create!(user_id: User.current.id, status: data['status'], status_order: StaffManagerStatus::ORDER[data["status"]], paused_at: Time.now)
        STDERR.puts "new_user_state: #{new_user_state} #{new_user_state.inspect}"
        unless new_user_state
          if data["status"] == 'paused' && RttaUserState.where(user_id: User.current.id).first.status != 'paused'
            STDERR.puts "not paused"
            RttaUserState.where(user_id: User.current.id)
                .update_all(status: data["status"], status_order: StaffManagerStatus::ORDER[data["status"]], updated_at: Time.now, paused_at: Time.now)
          else
            STDERR.puts "paused"
            RttaUserState.where(user_id: User.current.id)
                .update_all(status: data["status"], status_order: StaffManagerStatus::ORDER[data["status"]], updated_at: Time.now)
          end
        end
      end
      'Status updated'
    rescue => e
      message = "Rtta Server. update status error: #{e.message}"
      Rails.logger.error message
      return {errors: [message]}
    end

    # begin
    #   # Find old or create new user state
    #   user_state = RttaUserState.where(user_id: User.current.id).first
    #   unless user_state
    #     user_state = RttaUserState.new(user_id: User.current.id)
    #   end
    #   user_state.issue_id = data.key?("issue_id") ? data["issue_id"] : nil
    #   user_state.status = data.key?("status") ? data["status"] : StaffManagerStatus::WORKING
    #   user_state.status_order = data.key?("status") ? StaffManagerStatus::ORDER[data["status"]] : StaffManagerStatus::ORDER[StaffManagerStatus::WORKING]
    #   user_state.paused_at = (data["status"] == StaffManagerStatus::PAUSED) ? Time.now : nil
    #
    #   if user_state.save
    #     user_state.touch
    #     render :json => {message: ['User state updated']}
    #   else
    #     Rails.logger.error "Rtta update_user_state error: #{user_state.errors.to_json}"
    #     STDERR.puts "Rtta update_user_state error: #{user_state.errors.to_json}"
    #     render :json => {errors: [user_state.errors.to_json]}
    #   end
    # rescue => e
    #   Rails.logger.error "Rtta update_user_state error: #{e.message}"
    #   STDERR.puts "Rtta update_user_state error: #{e.message}"
    #   render :json => {errors: [e.message]}
    # end
  end

  def custom_field_values_changed?(time_entry, values)
    # return lalse for 3.1.6.1
    return false


    values = values.stringify_keys
    changed = false

    # custom_fields = time_entry.available_custom_fields.map {|cf| [cf.id, cf]}.to_h
    custom_fields = time_entry.available_custom_fields.each_with_object({}) do |cf, hash|
      hash[cf.id] = cf
    end

    time_entry.custom_field_values.each do |custom_field_value|
      key = custom_field_value.custom_field_id.to_s

      if values.has_key?(key)
        # Fix conversion [""] to nil
        if values[key] == nil
          values[key] = [""]
        end
        if custom_field_value.value != values[key]
          STDERR.puts "changed 1"
          changed = true
          break
        end
      else # check if saved value doesn't match default value
        # if custom_fields[key.to_i].field_format == 'list' && custom_fields[key.to_i].multiple
        #   default_value = [""]
        # else
        #   default_value = custom_fields[key.to_i].default_value
        # end
        if custom_fields[key.to_i].field_format == 'list' && custom_fields[key.to_i].multiple
          if (custom_field_value.value != custom_fields[key.to_i].default_value) && (custom_field_value.value != [""]) && (custom_field_value.value != [])
            changed = true
            break
          end
        else
          if custom_field_value.value != custom_fields[key.to_i].default_value
            STDERR.puts "changed 3"
            changed = true
            break
          end
        end
      end
    end
    changed
  end

  def get_custom_fields
    begin
      # custom_fields = CustomField.where(type: "TimeEntryCustomField")
      # return {list: custom_fields.as_json(:methods => [:type])}
      return {list: Array.new}
    rescue StandardError => e
      message = "Rtta Server. get custom fields error: #{e.message}"
      Rails.logger.error message
      return {errors: [message]}
    end
  end
end
