require_relative '../../inc/ukrsol/server/rtt_plugin_server'

class TimeTrackerAdministratorController < ApplicationController
  # unloadable
  ActiveRecord::Base.include_root_in_json = false

  if Redmine::VERSION::MAJOR <= 2
    before_filter :check_permissions, :except => [:issue_statuses, :issue_priorities, :rtt_settings]
    before_filter :check_if_user_logged_in, :except => [:rtt_settings]
  else
    before_action :check_permissions, :except => [:issue_statuses, :issue_priorities, :rtt_settings]
    before_action :check_if_user_logged_in, :except => [:rtt_settings]
    self.main_menu = false
  end

  def index
    start_server
  end

  def menu_item

  end

  def check_permissions
    deny_access unless User.current.admin
    # check if user has at least one role allowed for this action
    # deny_access unless User.current.allowed_to?(:view_staff_manager, nil, :global => true)
  end

  def check_if_user_logged_in
    deny_access unless User.current.status == 1
  end

  def stop_server
    $rtta_server.close
  end

  def start_server

    # Do not start server if plugin disabled
    unless Setting.plugin_redmine_time_tracker_administrator['plugin_enabled']
      return
    end

    settings = Setting.plugin_redmine_time_tracker_administrator
    # socket_port = settings["socket_port"]

    # TODO: temporary hardcode socket port value
    socket_port = 47123

    # WORKING VARIANT !!!
    Thread.new do
      begin
        if defined?($rtta_server) && $rtta_server != nil
          ::Ukrsol::Server::RttPluginServer.instance.run('0.0.0.0', socket_port)
        else
          $rtta_server = ::Ukrsol::Server::RttPluginServer.instance
          $rtta_server.set_auth_url request.base_url+rtta_events_authenticate_path
          $rtta_server.run('0.0.0.0', socket_port)
        end
      rescue StandardError => e
        Thread.exit
      end
    end
  end

  ##############################
  #Plugin API
  ##############################
  def active_users_count
    begin
      connections = SocketClient.new("127.0.0.1", 47123)
        .receive({
           event: 'ss_get_users_connections_count',
           message: {}
         })

      render :json => {active_users_count: connections}
    rescue => e
      message = "RTTA. Server get connections error: #{e.message}"
      render :json => {message: message}
    end
  end

  def plugin_settings
    start_server

    begin
      render :json =>  {plugin_settings: Setting.plugin_redmine_time_tracker_administrator}
    rescue => e
      message = "RTTA. Get plugin settings error: #{e.message}"
      Rails.logger.error message
      render :json => {message: message}
    end
  end

  # Save standard settings
  def save_plugin_settings
    begin
      # Check if params sent
      if params.key?(:settings)
        setting = (Redmine::VERSION::MAJOR <= 2) ? params[:settings] : params[:settings].permit!.to_h
        @plugin = Redmine::Plugin.find('redmine_time_tracker_administrator')
        Setting.send "plugin_#{@plugin.id}=", setting
        message = 'Plugin settings saved'
      else
        message = 'Plugin settings not sent'
      end

      render :json => {message: message}
    rescue => e
      message = "RTTA. Save plugin settings error: #{e.message}"
      Rails.logger.error message
      render :json => {message: message}
    end

    begin
      # Close server if plugin disabled
      if !Setting.plugin_redmine_time_tracker_administrator['plugin_enabled'] && defined?($rtta_server)
        $rtta_server.close
      end
    rescue => e
    end
  end

  def rtt_settings
    begin
      settings = RttaSetting.where(item_type: params[:type])

      # If no settings found, add settings for current user
      if settings.length == 0 && params[:type] === 'user'
        settings = [{
          id: nil,
          item_id: User.current.id,
          item_type: "user",
          params: '{}',
          item: User.current,
        }]

        render :json => settings
      else
        render :json => settings.as_json(:methods => [:item])
      end
    rescue => e
      message = "RTTA. Get rtt settings error: #{e.message}"
      Rails.logger.error message
      render :json => {message: message}
    end
  end

  # Save rtt settings
  def save_rtt_settings
    begin
      # Save Time Tracker Settings
      if params[:rttaSettingsItems] && params[:type]
        RttaSetting::save_rtta_settings(params[:rttaSettingsItems], params[:type])
      end

      render :json => {message: 'Rtt settings saved'}
    rescue => e
      message = "RTTA. Save rtt settings error: #{e.message}"
      Rails.logger.error message
      render :json => {message: message}
    end

    begin
      # Broadcast settings update event
      SocketClient.new("127.0.0.1", 47123)
          .send({
                    event: 'broadcast_to_users',
                    message: {event: 'WS_APP_SETTINGS', message: ''}
                })
    rescue => e
    end
  end

  def users
    begin
      limit = params[:limit].present? ? params[:limit] : 10
      offset = params[:offset].present? ? params[:offset] : 0

      users = User.status(1)
      users = users.where(
          "CONCAT(#{User.table_name}.firstname, ' ', #{User.table_name}.lastname) LIKE ? OR users.login LIKE ?",
          "%#{params[:q]}%",
          "%#{params[:q]}%"
      ) if params[:q] != ''
      users = users.order("#{User.table_name}.firstname asc")
      users = users.limit(limit).offset(offset)

      ActiveRecord::Base.include_root_in_json = false
      users_json = users.all.as_json

      # sql = users.to_sql
      # render :json => {:users => users_json, :sql => sql}
      render :json => users_json
    rescue => e
      message = "RTTA. Get users error: #{e.message}"
      Rails.logger.error message
      render :json => {message: message}
    end
  end

  # Get issue statuses
  def issue_statuses
    begin
      issue_statuses = IssueStatus.sorted.to_a

      if issue_statuses
        render :json => {:issue_statuses => issue_statuses}
      else
        render_404
      end
    rescue
      message = "RTTA. Get issue statuses error: #{e.message}"
      Rails.logger.error message
      render :json => {message: message}
    end
  end

  def issue_priorities
    begin
      klass = Enumeration.get_subclass('issue_priorities')

      if klass
        enumerations = klass.shared.sorted.to_a
        render :json => {:issue_priorities => enumerations}
      else
        render_404
      end
    rescue
      message = "RTTA. Get issue priorities error: #{e.message}"
      Rails.logger.error message
      render :json => {message: message}
    end
  end
end
