class TimeTrackerStuffManagerController < ApplicationController
  ActiveRecord::Base.include_root_in_json = false

  if Redmine::VERSION::MAJOR <= 2
    before_filter :check_permissions, :except => []
  else
    before_action :check_permissions, :except => []
    self.main_menu = false
  end

  def index
  end

  def check_permissions
    # deny_access unless User.current.admin
    # check if user has at least one role allowed for this action
    deny_access unless User.current.allowed_to?(:view_staff_manager, nil, :global => true)
  end

  def roles
    roles = Role.sorted

    ActiveRecord::Base.include_root_in_json = false
    render :json => roles
  end

  def groups
    groups = Group.sorted
    # Translate groups names to redmine language
    groups.each do |group|
      group.lastname = group.to_s
    end

    ActiveRecord::Base.include_root_in_json = false
    render :json => groups
  end

  def users
    if request.post?
      limit = params[:limit].present? ? params[:limit] : 10
      offset = params[:offset].present? ? params[:offset] : 0

      begin
        RttaUserState.update_user_states_order

        users = User.status(1).rtta_belongs_to_projects(Project.visible(User.current).pluck(:id))
        users = users.rtta_in_groups(params[:group_id]) if params[:group_id] && !params[:group_id].empty? && !params[:group_id].include?('all')
        users = users.rtta_belongs_to_role(params[:role_id]) if params[:role_id] && !params[:role_id].empty? && !params[:role_id].include?('all')

        users = users.select("DISTINCT #{User.table_name}.*, #{RttaUserState.table_name}.issue_id, #{RttaUserState.table_name}.project_id, #{RttaUserState.table_name}.status_order, #{Project.table_name}.name, #{Issue.table_name}.subject, #{Project.table_name}.name IS NULL, #{Issue.table_name}.subject IS NULL, #{RttaUserState.table_name}.status_order IS NULL")
        users = users.joins("LEFT OUTER JOIN #{RttaUserState.table_name} ON #{RttaUserState.table_name}.user_id = #{User.table_name}.id LEFT OUTER JOIN #{Issue.table_name} ON #{Issue.table_name}.id = #{RttaUserState.table_name}.issue_id LEFT OUTER JOIN #{Project.table_name} ON #{Project.table_name}.id = #{Issue.table_name}.project_id")
        users = users.where("CONCAT(#{User.table_name}.firstname, ' ', #{User.table_name}.lastname) LIKE '%#{params[:filters][:userName]}%'") if params[:filters][:userName] != ''

        if params[:filters][:issueSubject] != '' || params[:filters][:projectName] != ''
          users = users.where(
              # "#{Issue.table_name}.subject LIKE ? OR #{Issue.table_name}.id LIKE ? OR CONCAT('#', #{Issue.table_name}.id) LIKE ?",
              "#{Issue.table_name}.subject LIKE ? OR cast(#{Issue.table_name}.id as CHAR) LIKE ? OR CONCAT('#', #{Issue.table_name}.id) LIKE ?",
              "%#{params[:filters][:issueSubject]}%",
              "%#{params[:filters][:issueSubject]}%",
              "%#{params[:filters][:issueSubject]}%"
          ) if params[:filters][:issueSubject] != ''
          users = users.where("#{Project.table_name}.name LIKE ?", "%#{params[:filters][:projectName]}%") if params[:filters][:projectName] != ''
        end

        if Redmine::VERSION::MAJOR <= 4
          users = users.order(sort_params)
        else
          users = users.order(Arel.sql(sort_params))
        end

        users = users.limit(limit).offset(offset)

        # sql = users.to_sql

        ActiveRecord::Base.include_root_in_json = false
        users_json = users.as_json(
            decimals: 2,
            :except => [:login, :hashed_password, :passwd_changed_on, :salt],
            :include => [
                :tracker_state => {
                    :include => [
                        :issue => {
                            :include => [
                                :project,
                                :priority,
                            ],
                            :methods => [:total_spent_hours],
                        },
                        # :project => {:except => [:lft]},
                    ],
                    :methods => [
                        :spent_hours_today,
                        :spent_hours_issue_today,
                        :spent_hours_project_all,
                        :total_spent_hours,
                        :estimated_hours,
                        :time_left,
                        :get_status,
                        :default_priority_id,
                        :_project,
                        # :avatar_url,
                    # :get_status_text,
                    ],
                },
            ],
            :methods => [
                :avatar_url,
            ],
        )

        # render :json => {:users => users_json, :sql => sql}
        render :json => {:users => users_json}
      rescue => e
        render :json => {:users => e.inspect, :backtrace => e.backtrace.join("\n")}
      end
    end
  end

  def sort_params
    sort_order = params[:sort][:order]

    case params[:sort][:column]
    when "username"
      sort_order == "desc" ? "#{User.table_name}.firstname DESC" : "#{User.table_name}.firstname ASC"
    when "project"
      sort_order == "desc" ? "#{Project.table_name}.name IS NULL, #{Project.table_name}.name DESC" : "#{Project.table_name}.name IS NULL, #{Project.table_name}.name ASC"
    when "issue"
      sort_order == "desc" ? "#{Issue.table_name}.subject IS NULL, #{Issue.table_name}.subject DESC" : "#{Issue.table_name}.subject IS NULL, #{Issue.table_name}.subject ASC"
    when "status"
      # sort_order == "desc" ? "#{RttaUserState.table_name}.status_order asc, #{User.table_name}.firstname asc" : "#{RttaUserState.table_name}.status_order desc, #{User.table_name}.firstname asc"
      sort_order == "desc" ? "#{RttaUserState.table_name}.status_order IS NULL DESC, #{RttaUserState.table_name}.status_order ASC, #{User.table_name}.firstname ASC" : "#{RttaUserState.table_name}.status_order IS NULL ASC, #{RttaUserState.table_name}.status_order DESC, #{User.table_name}.firstname ASC"
    else
      # sort_order == "desc" ? "#{User.table_name}.firstname desc" : "#{User.table_name}.firstname asc"
      # sort_order == "desc" ? "#{RttaUserState.table_name}.status_order ASC, #{User.table_name}.firstname ASC" : "#{RttaUserState.table_name}.status_order DESC, #{User.table_name}.firstname ASC"
      sort_order == "desc" ? "#{RttaUserState.table_name}.status_order IS NULL DESC, #{RttaUserState.table_name}.status_order ASC, #{User.table_name}.firstname ASC" : "#{RttaUserState.table_name}.status_order IS NULL ASC, #{RttaUserState.table_name}.status_order DESC, #{User.table_name}.firstname ASC"
    end
  end

  # def update_user_states_order
  #   actual_time = Time.now - StaffManagerStatus::OFFLINE_TIME.minutes
  #   # RttaUserState.where("updated_at < ?", actual_time).update_all(status_order: StaffManagerStatus::ORDER['offline'])
  #   RttaUserState.where("updated_at < ?", actual_time)
  #       .update_all(status_order: StaffManagerStatus::ORDER['offline'], status: StaffManagerStatus::OFFLINE)
  # end
end