class RttaSetting < ActiveRecord::Base

  include Redmine::SafeAttributes

  def item
    if item_type === 'user'
      User.find(item_id)
    else
      nil
    end
  end

  def to_rtt_app_format
    params_hash = JSON.parse params
    data = {settings: {}, read_only_state: {}}
    params_hash.each do |param_name, param_data|
      data[:settings][param_name] = param_data['value']
      data[:read_only_state][param_name] = param_data['readonly']
    end
    data
  end

  def self.save_rtta_settings(items, type = 'default')
    STDERR::puts "Save Items"
    case type
    when 'default'
      self.save_default_settings(items[0])
    when 'user'
      self.save_users_settings(items)
    else
      #
    end
  end

  def self.save_users_settings(users_items)
    # Update users settings
    users_items.each do |user|
      # Check if settings for user exists
      # user_settings = RttaSetting.find_by(item_type: 'user', item_id: user['id'])
      user_settings = RttaSetting.where(item_type: 'user', item_id: user['id']).first
      # update existed
      if user_settings
        if Redmine::VERSION::MAJOR <= 4
          user_settings.update_attributes(params: user['settings'].to_json)
        else
          user_settings.update(params: user['settings'].to_json)
        end
      else
        # insert new
        RttaSetting.create(item_id: user['id'], item_type: 'user', params: user['settings'].to_json)
      end

      # Delete all other users
      users_ids = users_items.map{|the_user| the_user['id']} # Get all users ids
      RttaSetting.where(item_type: 'user').where('item_id NOT IN (?)', users_ids).delete_all
    end
  end

  def self.save_default_settings(item)
    # Check if default settings exits
    # default_settings = RttaSetting.find_by(item_type: 'default')
    default_settings = RttaSetting.where(item_type: 'default').first
    # update existed
    if default_settings
      if Redmine::VERSION::MAJOR <= 4
        default_settings.update_attributes(params: item['settings'].to_json)
      else
        default_settings.update(params: item['settings'].to_json)
      end
    else
      # insert new
      RttaSetting.create(item_id: 0, item_type: 'default', params: item['settings'].to_json)
    end
  end

  def params=(params)
    super(params)
  end

  def params
    # Get saved params
    params_hash = JSON.parse(super)
    # Get actual issue priorities
    klass = Enumeration.get_subclass('issue_priorities')
    if klass
      issue_priorities = klass.shared.sorted.to_a
    else
      issue_priorities = []
    end

    # Index issue priorities by id
    indexed_priorities = Hash[issue_priorities.map { |r| [r[:id], r] }]

    # Unset priority what not exists yet
    params_hash['PRIORITIES_ORDER']["value"].each do |priority_id|
      params_hash['PRIORITIES_ORDER']["value"].delete(priority_id) unless indexed_priorities.key?(priority_id)
    end

    # Add new priorities to the end of the list
    indexed_priorities.each do |priority_id, priority|
      params_hash['PRIORITIES_ORDER']["value"] << (priority.id) unless params_hash['PRIORITIES_ORDER']["value"].include?(priority.id)
    end

    params_hash.to_json
  end
end
