require 'digest/md5'
require 'cgi'

class RttaUserData < ActiveRecord::Base
  self.table_name = 'rtta_users_data'

  DEFAULT_OPTIONS = {
      # The URL of a default image to display if the given email address does
      # not have a gravatar.
      :default => 'mm',

      # The default size in pixels for the gravatar image (they're square).
      :size => 50,

      # The maximum allowed MPAA rating for gravatars. This allows you to
      # exclude gravatars that may be out of character for your site.
      :rating => 'PG',

      # Whether or not to display the gravatars using HTTPS instead of HTTP
      :ssl => false,
  }

  ADDITION_FIELDS = %w(t a n s)

  def self.get_user_data
    result = {}
    self.where(user_id: User.current.id).each do |user_data|
      result[user_data.param] = user_data.value === "null" ? {} : JSON.parse(user_data.value)
    end

    # Add additional info
    if result.key?("users") && result["users"].kind_of?(Array)
      result["users"].each_with_index do |user_data, i|
        user = User.find_by_id(user_data["id"])
        # Add extra info
        if user
          ADDITION_FIELDS.each do |field|
            case field
            when 't'
              if user.tracker_state.any?
                user.tracker_state.each do |tracker_state|
                  if tracker_state.issue_id
                    user_data['t'] = {} unless user_data['t'].present?
                    user_data['t'][tracker_state.issue_id] = {
                        i_id: tracker_state.issue_id,
                        i_n: tracker_state.issue.subject,
                        p_id: tracker_state.issue.project.id,
                        p_n: tracker_state.issue.project.name,
                        ip_id: tracker_state.issue.priority_id,
                    }
                  end
                end
              end
            when 'a'
              user_data['a'] = self.avatar_url(user.mail.to_s.downcase, {default: '404'})
            when 'n'
              user_data['n'] = user.name(:firstname_lastinitial)
            end
          end
        else
          # User not found, delete if from data
          result["users"].delete_at(i)
        end
      end
    end
    result
  end

  def self.clear_user_data(user_data)
    if user_data.kind_of?(Array)
      user_data.each do |user|
        ADDITION_FIELDS.each do |field|
          user.delete(field) if user.key?(field)
        end
      end
    end
    user_data
  end

  # Return the gravatar URL for the given email address.
  def self.avatar_url(email, options={})
    email_hash = Digest::MD5.hexdigest(email)
    options = DEFAULT_OPTIONS.merge(options)
    options[:default] = CGI::escape(options[:default]) unless options[:default].nil?
    self.gravatar_api_url(email_hash).tap do |url|
      opts = []
      [:rating, :size, :default].each do |opt|
        unless options[opt].nil?
          value = options[opt]
          opts << [opt, value].join('=')
        end
      end
      url << "?#{opts.join('&')}" unless opts.empty?
    end
  end

  # Returns the base Gravatar URL for the given email hash
  def self.gravatar_api_url(hash)
    "https://www.gravatar.com/avatar/#{hash}"
  end
end
