include ActionView::Helpers::DateHelper

class RttaUserState < ActiveRecord::Base
# class TimeManagerUserState < ApplicationRecord
  include Redmine::SafeAttributes

  belongs_to :issue
  belongs_to :project
  belongs_to :user

  # validates_presence_of :issue_id
  # validates_inclusion_of :status, :in => %w(working paused)
  validates :status, inclusion: { in: %w(working paused offline), message: "%{value} is not a valid status" }

  scope :user_working_states, lambda {|user, user_state|
    actual_time = Time.now - StaffManagerStatus::OFFLINE_TIME.minutes

    where(:user_id => user.id)
      .where("id <> ?", user_state.id)
      .where("updated_at >= ?", actual_time)
      .where(status: 'working')
  }

  def estimated_hours
    if project
      cond = project.project_condition(Setting.display_subprojects_issues?)
      "%0.2f" % Issue.visible.where(cond).sum(:estimated_hours).to_f
    else
      (issue != nil && issue.estimated_hours) ? ("%0.2f" % [issue.estimated_hours]) : nil
    end
  end

  def total_spent_hours
    if project
      project = Project.where(id: project_id).first
      cond = project.project_condition(Setting.display_subprojects_issues?)
      # TimeEntry.visible.where(cond).sum(:hours).to_f.round(2)
      "%0.2f" % TimeEntry.visible.where(cond).sum(:hours).to_f.round(2)
    else
      issue ? "%0.2f" % [issue.total_spent_hours] : nil
    end
  end

  def spent_hours_today
    if issue == nil
      return nil
    end

    if issue.leaf?
       time_spent_today = issue.time_entries.where(
          time_entries: {
            spent_on: Time.now.strftime("%Y-%m-%d"),
            user_id: user.id
          },
       ).sum(:hours) || 0.0
    else
      time_spent_today = issue.self_and_descendants.joins(:time_entries).where(
        time_entries: {
         spent_on: Time.now.strftime("%Y-%m-%d"),
         user_id: user.id
        },
      ).sum("#{TimeEntry.table_name}.hours").to_f || 0.0
    end

    "%0.2f" % [time_spent_today]
  end

  def spent_hours_issue_today
    "%0.2f" % [user.time_entries.where(
      time_entries: {spent_on: Time.now.strftime("%Y-%m-%d")},
    ).sum("#{TimeEntry.table_name}.hours").to_f || 0.0]
  end

  def spent_hours_project_all
    if issue == nil
      return nil
    end

    "%0.2f" % [user.time_entries.where(
      spent_on: Time.now.strftime("%Y-%m-%d"),
      project_id: issue.project_id,
    ).sum("#{TimeEntry.table_name}.hours").to_f || 0.0]
  end

  def time_left
    if issue == nil
      return nil
    end

    time_left = issue.estimated_hours ? (issue.estimated_hours - issue.total_spent_hours) : nil
    if time_left == nil
      nil
    else
      "%0.2f" % [time_left]
    end
  end

  def get_status
    # # Treat user offline after 5 min inactivity
    # offline_check_time = StaffManagerStatus::OFFLINE_TIME*60
    # now_time = Time.now
    # offline_time = now_time - updated_at
    #
    # if offline_time > offline_check_time
    #   status_order = 40
    #   'offline'
    # else
    #   status
    # end

    # Treat user offline after 5 min inactivity
    if status_order == StaffManagerStatus::ORDER['offline']
      'offline'
    else
      status
    end
  end

  def default_priority_id
    IssuePriority.default
  end

  def _project
    Project.where(id: project_id).first
  end

  # def avatar_url
  #   RttaUserData::avatar_url(user.mail)
  # end

  def self.update_user_states_order
    actual_time = Time.now - StaffManagerStatus::OFFLINE_TIME.minutes
    self.where("updated_at < ?", actual_time)
        .update_all(status_order: StaffManagerStatus::ORDER['offline'], status: StaffManagerStatus::OFFLINE)
  end
end
