require "socket"
require "thread"
require_relative '../../imanel/websocket'

module Ukrsol
  module Server
    class Base
      RECV_LENGTH = 1024

      attr_accessor :server, :connections
      # def initialize(ip, port)
      def initialize

        # @ip = ip
        # @port = port
        @connections = Hash.new

        # begin
        #   @server = TCPServer.new(ip, port)
        #     # rescue Errno::EADDRINUSE, Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
        # rescue => e
        #   return false
        # end
        # run
      end

      def close
        @server.close if defined?(@server)
        @connections.each {|connection_object_id, connection_data| connection_data[:socket].close}
      end

      def run(ip, port)
        begin
          @server = TCPServer.new(ip, port)

        rescue => e
          return false
        end

        loop {

          # for each user connected and accepted by server, it will create a new thread object
          # and which pass the connected client as an instance to the block
          Thread.start(@server.accept) do | client |
          # Thread.start(@sec_server.accept) do | client |
            begin

              http_request = ''
              while (line = client.gets) && (line != "\r\n")
                # Rails.logger.info "users_data: http_request line: #{line}"
                http_request += line
              end
              http_request += "\r\n"


              # Check if pure socket connection, process it and exit thread.
              if http_request.match(/simple_socket/)
                if should_accept_ss_connection?(client, http_request)
                  Rails.logger.error "users_data: ss connection accepted"
                  on_message(client, http_request)
                end
                client.close
                Rails.logger.error "users_data: ss connection closed"
                ActiveRecord::Base.connection.close
                Thread.exit
              end

              # Check if WebSocket connection, if not - exit thread
              unless http_request.match(/^Sec-WebSocket-Key: (\S+)/)
                client.close
                ActiveRecord::Base.connection.close
                Thread.exit
              end

              handshake = WebSocket::Handshake::Server.new
              handshake << http_request

              unless handshake.finished? && handshake.valid?
                Rails.logger.error "Rtta Server: Handshake error"
                client.close
                ActiveRecord::Base.connection.close
                Thread.exit
              end

              # Get headers
              params = parse_htt_params(http_request)
              headers = parse_http_headers(http_request)

              unless should_accept_connection?(client, headers, params)
                client.close
                ActiveRecord::Base.connection.close
                Thread.exit
              end

              # Respond with accept websocket headers
              client.write handshake.to_s

              @connections[client.object_id] = {
                  :socket => client,
                  :handshake => handshake,
                  :semaphore => Mutex.new,
              }

              on_connection_accepted(client)
              check_connection_state(client, handshake)
              listen_client_messages(client, handshake)

              ActiveRecord::Base.connection.close
              Thread.exit
            rescue => e
            end
          end
        }
      end

      def should_accept_connection?(client, headers, params)
        true
      end

      def on_connection_accepted(client)
      end

      def should_accept_ss_connection?(client, message)
        true
      end

      def listen_client_messages(client, handshake)
        frame = WebSocket::Frame::Incoming::Server.new(:version => handshake.version)

        # get client messages while it is in @connections
        while @connections.key?(client.object_id) do
          begin
            frame << client.recv(RECV_LENGTH)
            # Frame complete and parsed
            if next_frame = frame.next
              if next_frame.type == :text
                on_message(client, next_frame.to_s)
              end
            end
          rescue Exception => e
          end
        end
      end

      def on_message(client, msg)
      end

      def send_message(client, msg)
        begin
          handshake = @connections[client.object_id][:handshake]
          client = @connections[client.object_id][:socket]
          frame = WebSocket::Frame::Outgoing::Server.new(:version => handshake.version, :data => msg, :type => :text)
          @connections[client.object_id][:semaphore].synchronize {
            client.write frame.to_s
          }
        rescue Exception => e
        end
      end

      def check_connection_state(client, handshake)
        #periodicaly send ping request in separate thread
        Thread.new do
          loop do
            unless sending_ping_successful?(client, handshake)
              on_before_connection_remove(client)
              disconnected_client = @connections.delete(client.object_id)
              on_disconnected(disconnected_client)
              Thread.exit
            end
            sleep 2
          end
        end
      end

      def on_before_connection_remove(client)

      end

      def on_disconnected(client)
      end

      def sending_ping_successful?(client, handshake)
        begin
          frame = WebSocket::Frame::Outgoing::Server.new(:version => handshake.version, :data => 'pong', :type => :ping)
          @connections[client.object_id][:semaphore].synchronize {
            client.write frame.to_s
          }
          return  true
        rescue Errno::EPIPE
          return false
        rescue StandardError => e
          return false
        end
      end

      def parse_http_headers(htt_request_str)
        htt_request_str.split($/).drop(1).inject({}) do |h, l|
          k, v = l.split(": ", 2)
          # h[k.downcase.gsub("-", "_").to_sym] = (v unless v.empty?)
          h[k.downcase.gsub("-", "_").to_sym] = (v.strip if v)
          h
        end
      end

      def parse_htt_params(http_request_str)
        method, full_path = http_request_str.split(' ')
        path, query = full_path.split('?')

        # return
        if query
          # CGI.parse(query).transform_values(&:first)
          Hash[CGI::parse(query).map{|k,v| [k, v.first]}]
        else
          Hash.new
        end
      end
    end
  end
end