require 'net/http'
require 'json'
require_relative 'base'
require 'singleton'

module Ukrsol
  module Server
    class RttPluginServer < Base
      include Singleton

      # @@the_instance = nil
      # def self.instance
      #   if @@the_instance
      #     @@the_instance
      #   else
      #     @@the_instance = self.new
      #   end
      # end

      RETURN_BACK_DATA_FIELD = 'request_id'
      CLEAN_FREQUENCY_FACTOR = 1000
      # def initialize(host, port, auth_url)
      def initialize

        # @authenticate_url = auth_url
        @connections_info = Hash.new
        @users_to_connections_ids_map = Hash.new
        @admin_connections = Array.new
        @admins_to_connections_ids_map = Hash.new
        ObjectSpace.define_finalizer(self, self.class.method(:finalize))
        # super(host, port)
        super
      end

      def set_auth_url(auth_url)
        @authenticate_url = auth_url
      end

      # def run(host, port, auth_url)
      #   @authenticate_url = auth_url
      #   super.run(host, port)
      # end

      def self.finalize(object_id)
      end

      def finalize(object_id)
      end

      def should_accept_connection?(client, headers, params)
        authenticated_user_info = authenticate_connection(headers, params)

        if authenticated_user_info && authenticated_user_info.key?('id')
          if authenticated_user_info['admin'] && params.key?('staff_manager')
            authenticated_user_info['is_app'] = false
            @admin_connections.push(client.object_id)
            if @admins_to_connections_ids_map.key?(authenticated_user_info['id']) && @admins_to_connections_ids_map[authenticated_user_info['id']].kind_of?(Array)
              @admins_to_connections_ids_map[authenticated_user_info['id']] << client.object_id
            else
              @admins_to_connections_ids_map[authenticated_user_info['id']] = [client.object_id]
            end
          else
            authenticated_user_info['is_app'] = true
            if @users_to_connections_ids_map.key?(authenticated_user_info['id']) && @users_to_connections_ids_map[authenticated_user_info['id']].kind_of?(Array)
              @users_to_connections_ids_map[authenticated_user_info['id']] << client.object_id
            else
              @users_to_connections_ids_map[authenticated_user_info['id']] = [client.object_id]
            end
          end

          @connections_info[client.object_id] = {
              :user_info => authenticated_user_info,
              :headers => headers,
          }

          true
        else
          false
        end
      end

      def should_accept_ss_connection?(client, msg)
        begin
          msg = JSON.parse msg
          # Find user with given credentials
          if msg.key?('login') && msg.key?('key')
            User.where(login: msg['login']).where(hashed_password: msg['key']).exists?
          else
            false
          end
        rescue JSON::ParserError => e
          false
        end
      end

      def on_connection_accepted(client)
        user_info = @connections_info[client.object_id][:user_info]
        message = {
            :event => 'WS_AUTHENTICATED',
            :message => {
                text: "Hello #{user_info['firstname']} #{user_info['lastname']}!",
                # active_connections: active_users_connections_count,
            }
        }

        send_message(client, message.to_json)
        broadcast_connections_count

        # Update user state
        if user_info['is_app']
          save_new_connection_state(user_info, StaffManagerStatus::PAUSED, StaffManagerStatus::ORDER['paused'])
        end

        # Close database connection
        ActiveRecord::Base.connection.close
      end

      def active_users_connections_count
        active_connections = 0
        @users_to_connections_ids_map.each {|k, v| active_connections += v.length}
        # @admins_to_connections_ids_map.each {|k, v| active_connections += v.length}
        active_connections
      end

      def on_message(client, msg)
        begin
          msg = JSON.parse(msg)
          if msg.key?('event') && msg.key?('message')
            case msg['event']
              when 'request_to_user'
                do_request_to_user(client, msg['message'])
              when 'request_to_manager'
                do_request_to_manager(client, msg['message'])
              when 'broadcast_to_users'
                do_request_to_all_users(client, msg['message'])
              when 'ss_get_users_connections_count'
                ss_send_connections_count(client, msg['message'])
              when 'ss_get_users_data'
                ss_send_users_data(client, msg['message'])
              when 'WS_TIME_UPDATE'
                update_ticket_time_handler(client, msg['message'])
            else
                msg['event'] = 'WS_UNKNOWN_EVENT'
                send_message(client, msg.to_json)
            end
          end
        rescue JSON::ParserError
          Rails.logger.error "Rtta Json parse error: #{msg}"
        end
      end

      def update_ticket_time_handler(client, params)
        begin
          h = params["spent_time"]
          h = (h.is_a?(String) ? (h.to_hours || h) : h)
          if h == 0
            result = {:time_entry => nil}
          else
            update_time_result = update_ticket_time(client, params)
            if update_time_result.key?(:data) && update_time_result[:data].is_a?(TimeEntry)
              total_time_spent = "%0.2f" % [update_time_result[:data].issue.total_spent_hours]
              result = {
                  :time_entry => update_time_result[:data].as_json,
                  :total_spent_time => total_time_spent.to_f,
                  :duplicate => update_time_result.key?(:duplicate)
              }
            else
              result = update_time_result[:data]
            end
          end

          # Send result back to client
          result[RETURN_BACK_DATA_FIELD] = params[RETURN_BACK_DATA_FIELD] if params.key?(RETURN_BACK_DATA_FIELD)

          send_message(client, {event: 'WS_TIME_UPDATE_RESULT', message: result}.to_json)

          # Clean request_ids approximately one per CLEAN_FREQUENCY_FACTOR times
          RttaAppRequest.clean_old if rand(CLEAN_FREQUENCY_FACTOR) == 0
        rescue StandardError => e
          Rails.logger.error "Rtta Server. update ticket time handler error: #{e}"
        end
      end

      def update_ticket_time(client, params)
        begin
          user_info = @connections_info[client.object_id][:user_info]
          user_current = User.where(id: user_info['id']).first
          issue = Issue.where(id: params["issue_id"]).first

          # Issue not found
          unless issue
            # return {errors: [l(:rtta_issue_not_found)]}
            return {data: {errors: ['Issue not found']}}
          end


          # Check if time was not added before (check request_id)
          existed_request_id_condition = RttaAppRequest
             .where(user_id: user_info["id"], issue_id: params["issue_id"])
             .where("rtta_app_requests.request_ids LIKE ?", "%#{params[RETURN_BACK_DATA_FIELD]}%")
          if existed_request_id_condition.exists?
            # Time already added, just return time entry without saving
            return {
              data: TimeEntry.order(id: :desc).where(
                spent_on: params["date"],
                issue_id: params["issue_id"],
                user_id: user_info["id"],
                comments: params["comment"],
                activity_id: params["activity_id"],
              ).first,
              duplicate: true
            }
          end

          # Try to find existed time entry
          time_entry = TimeEntry.order(id: :desc).where(
              spent_on: params["date"],
              issue_id: params["issue_id"],
              # user_id: User.current.id,
              user_id: user_info["id"],
              comments: params["comment"],
              activity_id: params["activity_id"],
          ).first

          # Update existed time entry or create new
          if time_entry
            h = params["spent_time"]
            begin
              time_entry.hours = time_entry.hours.to_f + (h.is_a?(String) ? (h.to_hours || h) : h)
            rescue TypeError => e
              # return {errors: [l(:staff_manager_incorrect_spent_time) % [issue.id]]}
              return {data: {errors: ['Incorrect spent time.' % [issue.id]]}}
            end
          else
            # Create new
            time_entry = TimeEntry.new(
                :issue => issue,
                :project => issue ? issue.project : nil,
                # :user => User.current,
                :user => user_current,
                # :user_id => user_info["id"],
                :spent_on => params["date"],
                :hours => params["spent_time"].to_f,
                :comments => params["comment"],
                :activity_id => params["activity_id"],
            )
          end

          # Check authorization
          if time_entry.project && !user_current.allowed_to?(:log_time, time_entry.project)
            # return {errors: [l(:rtta_not_allowed_add_time_entry) % [issue.id]]}
            return {data: {errors: ['You are not allowed to add time to task #%s. Contact redmine administrator to get access.' % [issue.id]]}}
          end

          # Save time entry and return response
          if time_entry.save
            # Save request data (request_id)
            save_request_data time_entry, params, user_info
            {data: time_entry}
          else
            {data: {errors: [time_entry.errors.to_json]}}
          end
        rescue StandardError => e
          Rails.logger.error "Rtta Server. Update time error: #{e}"
        end
      end

      def save_request_data(time_entry, params, user_info)
        # Find existed data. Or create new
        rtta_app_request = RttaAppRequest.where(user_id: user_info["id"], issue_id: time_entry.issue.id).first
        rtta_app_request = RttaAppRequest.new(
          user_id: user_info["id"],
          issue_id: time_entry.issue.id,
          # request_ids: [params[RETURN_BACK_DATA_FIELD]].to_json,
        ) unless rtta_app_request

        # Add request id to array
        rtta_app_request.add_request_id params[RETURN_BACK_DATA_FIELD]
        rtta_app_request.save
      end

      def do_request_to_user(client, data)
        user_id = data['user_id']

        #find given user connection
        unless @users_to_connections_ids_map.key?(user_id)
          send_connection_not_found(client)
          return
        end

        if data.key?('message')
          data['message']['manager_id'] = client.object_id
        else
          data['message'] = {'manager_id' => client.object_id}
        end

        # return back for testing
        send_message(client, data.to_json)

        @users_to_connections_ids_map[user_id].each do |client_id|
          send_message(@connections[client_id][:socket], data.to_json) if @connections.key?(client_id)
        end
      end

      def do_request_to_all_users(client, data)
        # send message to all users
        @users_to_connections_ids_map.each do |user_id, user_clients_ids|
        # @admins_to_connections_ids_map.each do |user_id, user_clients_ids|
          user_clients_ids.each do |client_id|
            send_message(@connections[client_id][:socket], data.to_json) if @connections.key?(client_id)
          end
        end
      end

      def do_request_to_all_admins(data)
        # send message to all admins
        @admins_to_connections_ids_map.each do |user_id, user_clients_ids|
          user_clients_ids.each do |client_id|
            send_message(@connections[client_id][:socket], data.to_json) if @connections.key?(client_id)
          end
        end
      end

      def do_request_to_manager(client, data)
        client_id = data['manager_id']
        #find given manager connection
        if @connections.key?(client_id)
          send_message(@connections[client_id][:socket], data.to_json)
        else @connections.key?(client_id)
          send_connection_not_found(client)
        end
      end

      def send_connection_not_found(client)
        send_message(client, {'event' => 'connection_not_found', 'message' => 'Connection not found!'}.to_json)
      end

      def authenticate_connection(headers, params)
        uri = URI(@authenticate_url + (params.key?('key') ? '?key='+params['key'] : ''))

        auth_response = Net::HTTP.start(uri.host, uri.port, :use_ssl => uri.scheme == 'https') {|http|
          req = Net::HTTP::Get.new(uri.request_uri)
          req['Origin'] = headers[:origin]
          req['Cookie'] = headers[:cookie]
          # Check for auth params
          req['Authorization'] = ('Basic ' + params['auth']) if params.key?('auth')

          http.request(req)
        }

        unless auth_response.code == '200'
          return false
        end

        begin
          user_info = JSON.parse(auth_response.body)
        rescue JSON::ParserError
          Rails.logger.error "Rtta Auth json parse error"
          return false
        end

        user_info
      end

      def on_before_connection_remove(client)
        begin
          user_info =  @connections_info[client.object_id][:user_info]

          # Update user state
          if user_info['is_app']
            save_new_connection_state(user_info, StaffManagerStatus::OFFLINE, StaffManagerStatus::ORDER['offline'])
          end

          user_id = user_info['id']
          @users_to_connections_ids_map[user_id].delete(client.object_id) if @users_to_connections_ids_map.key?(user_id)
          @admins_to_connections_ids_map[user_id].delete(client.object_id) if @admins_to_connections_ids_map.key?(user_id)
          @connections_info.delete(client.object_id)
        rescue StandardError => e
        end

        broadcast_connections_count
      end

      def broadcast_connections_count
        message = {
            :event => 'WS_CONNECTIONS_COUNT',
            :message => {
                active_connections: active_users_connections_count,
            }
        }
        do_request_to_all_admins(message)
      end

      def ss_send_connections_count(client, data)
        client.write(active_users_connections_count.to_s)
      end

      def ss_send_users_data(client, data)
        client.write(@users_to_connections_ids_map.to_json + "\r\n\r\n")
      end

      def save_new_connection_state(user_info, status, status_order)
        begin
          # Find old or create new user state
          state = RttaUserState.where(user_id: user_info['id']).first
          unless state
            state = RttaUserState.new(user_id: user_info['id'])
          end

          # Update user status
          state.status = status
          state.status_order = status_order

          if state.save
            state.touch
          else
            Rails.logger.error "RTTA server state save error #{user_rtt_state.errors.to_json}"
          end
        rescue => e
          Rails.logger.error "RTTA server state save error! #{e.message}"
        end
      end
    end
  end
end
