require 'user'

module RttaUserPatch
  def self.included(base) # :nodoc:
    base.extend(ClassMethods)

    base.send(:include, InstanceMethods)

    # Same as typing in the class
    base.class_eval do
      unloadable # Send unloadable so it will not be unloaded in development
      scope :rtta_belongs_to_projects, lambda { |arg|
        # joins(:projects).where(projects: arg)
        joins(:members).where(:members => {:project_id => arg})
      }
      scope :rtta_belongs_to_role, lambda { |role_id|
        joins(members: [:member_roles]).where(:member_roles => {:role_id => role_id}) if role_id != 0
      }
      scope :rtta_in_groups, lambda {|groups|
        # group_id = group.is_a?(Group) ? group.id : group.to_i
        # where("#{User.table_name}.id IN (SELECT gu.user_id FROM #{table_name_prefix}groups_users#{table_name_suffix} gu WHERE gu.group_id = ?)", group_id)
        where("#{User.table_name}.id IN (SELECT gu.user_id FROM #{table_name_prefix}groups_users#{table_name_suffix} gu WHERE gu.group_id IN (?))", groups)
      }

      has_many :tracker_state, :class_name => 'RttaUserState'
      has_many :time_entries
    end
  end

  module ClassMethods

  end

  module InstanceMethods
    def managed_users
      # user_visible_project_ids = visible_project_ids
      user_visible_project_ids = Project.visible(self).pluck(:id)
      # User.rtta_belongs_to_projects(user_visible_project_ids).distinct
      User.rtta_belongs_to_projects(user_visible_project_ids)
    end

    def staff_manager
      self.allowed_to?(:view_staff_manager, nil, :global => true)
    end

    def managed_users_by_role(role_id)

    end

    def avatar_url
      RttaUserData::avatar_url(mail, size: 48)
    end
  end
end
User.send(:include, RttaUserPatch)
